From 5501c0632f323bde472ff94a25ed85bf05f4c1ca Mon Sep 17 00:00:00 2001
From: Gerard Vermeulen <gerard.vermeulen@posteo.net>
Date: Sat, 30 Dec 2023 19:25:25 +0100
Subject: [PATCH] org-babel-demarcate-block: duplicate switches too

* lisp/ob-babel.el (org-babel-demarcate-block): duplicate switches when
the demarcation duplicates headers.
* testing/lisp/test-ob.el (test-ob/demarcate-block-split): New test
for block splitting by demarcation.  It checks that switches and
header arguments (with the exception of multi-line header arguments)
are duplicated.
---
 lisp/ob-core.el         |  3 +++
 testing/lisp/test-ob.el | 37 +++++++++++++++++++++++++++++++++++++
 2 files changed, 40 insertions(+)

diff --git a/lisp/ob-core.el b/lisp/ob-core.el
index f7e4e255f..634b7626b 100644
--- a/lisp/ob-core.el
+++ b/lisp/ob-core.el
@@ -2078,6 +2078,7 @@ block of the same language with the previous."
            (save-excursion
              (goto-char place)
              (let ((lang (nth 0 info))
+                   (switches (nth 3 info))
                    (indent (make-string (org-current-text-indentation) ?\s)))
 	       (when (string-match "^[[:space:]]*$"
                                    (buffer-substring (line-beginning-position)
@@ -2089,6 +2090,8 @@ block of the same language with the previous."
 		        (if arg stars indent) "\n"
 		        indent (if upper-case-p "#+BEGIN_SRC " "#+begin_src ")
 		        lang
+                        (if (> (length switches) 1)
+                            (concat " " switches) switches)
 		        (if (> (length headers) 1)
 			    (concat " " headers) headers)
 		        (if (looking-at "[\n\r]")
diff --git a/testing/lisp/test-ob.el b/testing/lisp/test-ob.el
index 42c77ca56..70d56d199 100644
--- a/testing/lisp/test-ob.el
+++ b/testing/lisp/test-ob.el
@@ -26,6 +26,43 @@
 (require 'org-table)
 (eval-and-compile (require 'cl-lib))
 
+(ert-deftest test-ob/demarcate-block-split ()
+  "Test duplication of headers and switches in demarcation block splitting."
+  (org-test-with-temp-text "
+#+header: :var edge=\"unduplicated\"
+#+header: :wrap \"src any-spanish -i -n\"
+#+begin_src any-english -i -n :var here=\"seen\"
+
+above split
+<point>
+below split
+
+#+end_src
+"
+    (let (above below)
+      (org-babel-demarcate-block)
+      (setq above (org-babel-get-src-block-info))
+      (org-babel-next-src-block)
+      (setq below (org-babel-get-src-block-info))
+      ;; unduplicated multi-line header arguments:
+      (should (equal '(edge . "unduplicated")
+                     (cdr (assq :var (cdr (nth 2 above))))))
+      (should (not (equal '(edge . "unduplicated")
+                          (cdr (assq :var (cdr (nth 2 below)))))))
+      (should (equal "src any-spanish -i -n"
+                     (cdr (assq :wrap (nth 2 above)))))
+      (should (not (equal "src any-spanish -i -n"
+                          (cdr (assq :wrap (nth 2 below))))))
+      ;; duplicated headers and switches:
+      (should (string= "any-english" (nth 0 above)))
+      (should (string= "any-english" (nth 0 below)))
+      (should (string= "above split" (org-trim (nth 1 above))))
+      (should (string= "below split" (org-trim (nth 1 below))))
+      (should (equal '(here . "seen") (cdr (assq :var (nth 2 above)))))
+      (should (equal '(here . "seen") (cdr (assq :var (nth 2 below)))))
+      (should (string= "-i -n" (nth 3 above)))
+      (should (string= "-i -n" (nth 3 below))))))
+
 (ert-deftest test-ob/indented-cached-org-bracket-link ()
   "When the result of a source block is a cached indented link it
 should still return the link."
-- 
2.42.0

